use tokio::try_join;

use crate::prelude::*;

#[derive(Debug, Deserialize)]
pub struct Params {
    folder_id: Option<Uuid>,
}

#[derive(Debug, Serialize)]
pub struct Response {
    folder_id: Uuid,
    files: Vec<db::file::FileWithoutParentId>,
    folders: Vec<db::folder::FolderWithoutParentId>,
}

pub async fn list(
    Query(params): Query<Params>,
    State(pool): State<Pool>,
    claims: Claims,
) -> Result<Json<Response>, StatusCode> {
    let folder_id = db::folder::get_by_id(params.folder_id, claims.user_id, &pool)
        .await
        .handle_internal()?
        .ok_or(StatusCode::NOT_FOUND)?;

    let (files, folders) = try_join!(
        db::file::get_files(folder_id, &pool),
        db::folder::get_folders(folder_id, &pool)
    )
    .handle_internal()?;

    Ok(Json(Response {
        folder_id,
        files,
        folders,
    }))
}
